@@ -14,6 +14,7 @@
 #
 
 import argparse
+import ddt
 from unittest import mock
 import uuid
 
@@ -35,7 +36,6 @@ class TestShare(manila_fakes.TestShare):
 
     def setUp(self):
         super(TestShare, self).setUp()
-
         self.shares_mock = self.app.client_manager.share.shares
         self.shares_mock.reset_mock()
 
@@ -56,6 +56,9 @@ class TestShare(manila_fakes.TestShare):
         self.share_types_mock = self.app.client_manager.share.share_types
         self.share_types_mock.reset_mock()
 
+        self.share_networks_mock = self.app.client_manager.share.share_networks
+        self.share_networks_mock.reset_mock()
+
         self.app.client_manager.share.api_version = api_versions.APIVersion(
             MAX_VERSION)
 
@@ -67,7 +70,16 @@ class TestShare(manila_fakes.TestShare):
             0)
         return shares
 
+    def setup_share_groups_mock(self):
+        self.share_group_mock = self.app.client_manager.share.share_groups
+        self.share_group_mock.reset_mock()
 
+        share_group = manila_fakes.FakeShareGroup.create_one_share_group()
+        self.share_group_mock.get = mock.Mock(return_value=share_group)
+        return share_group
+
+
+@ddt.ddt
 class TestShareCreate(TestShare):
 
     def setUp(self):
@@ -106,7 +118,6 @@ class TestShareCreate(TestShare):
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
         columns, data = self.cmd.take_action(parsed_args)
 
         self.shares_mock.create.assert_called_with(
@@ -227,20 +238,24 @@ class TestShareCreate(TestShare):
         arglist = [
             self.new_share.share_proto,
             str(self.new_share.size),
-            '--share-type', self.share_type.id,
             '--snapshot-id', self.share_snapshot.id
 
         ]
         verifylist = [
             ('share_proto', self.new_share.share_proto),
             ('size', self.new_share.size),
-            ('share_type', self.share_type.id),
             ('snapshot_id', self.share_snapshot.id)
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        columns, data = self.cmd.take_action(parsed_args)
+        with mock.patch(
+                'manilaclient.common.apiclient.utils.find_resource',
+                mock.Mock(return_value=self.share_snapshot)):
+            columns, data = self.cmd.take_action(parsed_args)
+
+            osc_shares.apiutils.find_resource.assert_called_once_with(
+                mock.ANY, self.share_snapshot.id)
 
         self.shares_mock.create.assert_called_with(
             availability_zone=None,
@@ -251,12 +266,11 @@ class TestShareCreate(TestShare):
             share_group_id=None,
             share_network=None,
             share_proto=self.new_share.share_proto,
-            share_type=self.share_type.id,
+            share_type=None,
             size=self.new_share.size,
             snapshot_id=self.share_snapshot.id,
             scheduler_hints={}
         )
-
         self.assertCountEqual(self.columns, columns)
         self.assertCountEqual(self.datalist, data)
 
@@ -359,6 +373,28 @@ class TestShareCreate(TestShare):
             self.cmd.take_action,
             parsed_args)
 
+    @ddt.data('None', 'NONE', 'none')
+    def test_create_share_with_the_name_none(self, name):
+        arglist = [
+            '--name', name,
+            self.new_share.share_proto,
+            str(self.new_share.size),
+            '--share-type', self.share_type.id,
+        ]
+        verifylist = [
+            ('name', name),
+            ('share_proto', self.new_share.share_proto),
+            ('size', self.new_share.size),
+            ('share_type', self.share_type.id),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(
+            exceptions.CommandError,
+            self.cmd.take_action,
+            parsed_args)
+
 
 class TestShareDelete(TestShare):
 
@@ -406,6 +442,26 @@ class TestShareDelete(TestShare):
         self.shares_mock.delete.assert_has_calls(calls)
         self.assertIsNone(result)
 
+    def test_share_delete_with_share_group(self):
+        shares = self.setup_shares_mock(count=1)
+        share_group = self.setup_share_groups_mock()
+
+        arglist = [
+            shares[0].name,
+            '--share-group', share_group['id']
+        ]
+        verifylist = [
+            ("share_group", share_group['id']),
+            ('shares', [shares[0].name])
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.shares_mock.delete.assert_called_with(
+            shares[0], share_group['id'])
+        self.assertIsNone(result)
+
     def test_share_delete_with_force(self):
         shares = self.setup_shares_mock(count=1)
 
@@ -425,6 +481,25 @@ class TestShareDelete(TestShare):
         self.shares_mock.force_delete.assert_called_once_with(shares[0])
         self.assertIsNone(result)
 
+    def test_share_delete_with_soft(self):
+        shares = self.setup_shares_mock(count=1)
+
+        arglist = [
+            '--soft',
+            shares[0].name,
+        ]
+        verifylist = [
+            ('soft', True),
+            ("share_group", None),
+            ('shares', [shares[0].name]),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.shares_mock.soft_delete.assert_called_once_with(shares[0])
+        self.assertIsNone(result)
+
     def test_share_delete_wrong_name(self):
         shares = self.setup_shares_mock(count=1)
 
@@ -567,6 +642,8 @@ class TestShareList(TestShare):
             'project_id': None,
             'user_id': None,
             'offset': None,
+            'is_soft_deleted': False,
+            'export_location': None,
             'name~': None,
             'description~': None,
         }
@@ -951,6 +1028,26 @@ class TestShareList(TestShare):
 
         self.assertEqual(data, tuple(cmd_data))
 
+    def test_list_share_soft_deleted_api_version_exception(self):
+        self.app.client_manager.share.api_version = api_versions.APIVersion(
+            "2.60")
+        arglist = [
+            '--soft-deleted',
+        ]
+        verifylist = [
+            ('soft_deleted', True),
+        ]
+
+        search_opts = self._get_search_opts()
+        search_opts['is_soft_deleted'] = True
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(
+            osc_exceptions.CommandError,
+            self.cmd.take_action,
+            parsed_args)
+
     def test_list_share_api_version_exception(self):
         self.app.client_manager.share.api_version = api_versions.APIVersion(
             "2.35")
@@ -994,11 +1091,9 @@ class TestShareShow(TestShare):
             ("share", self._share.id)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
         cliutils.convert_dict_list_to_string = mock.Mock()
         cliutils.convert_dict_list_to_string.return_value = dict(
             self._export_location)
-
         columns, data = self.cmd.take_action(parsed_args)
         self.shares_mock.get.assert_called_with(self._share.id)
 
@@ -1017,7 +1112,7 @@ class TestShareSet(TestShare):
         super(TestShareSet, self).setUp()
 
         self._share = manila_fakes.FakeShare.create_one_share(
-            methods={"reset_state": None}
+            methods={"reset_state": None, "reset_task_state": None}
         )
         self.shares_mock.get.return_value = self._share
 
@@ -1173,6 +1268,22 @@ class TestShareSet(TestShare):
         self.assertRaises(
             osc_exceptions.CommandError, self.cmd.take_action, parsed_args)
 
+    def test_share_set_task_state(self):
+        new_task_state = 'migration_starting'
+        arglist = [
+            self._share.id,
+            '--task-state', new_task_state
+        ]
+        verifylist = [
+            ('share', self._share.id),
+            ('task_state', new_task_state)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self._share.reset_task_state.assert_called_with(new_task_state)
+        self.assertIsNone(result)
+
 
 class TestShareUnset(TestShare):
 
@@ -1468,7 +1579,6 @@ class TestAdoptShare(TestShare):
             ('protocol', 'NFS'),
             ('export_path', '10.0.0.1:/example_path')
         ]
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         columns, data = self.cmd.take_action(parsed_args)
         self.shares_mock.manage.assert_called_with(
@@ -1762,7 +1872,6 @@ class TestShowShareProperties(TestShare):
             }
         )
         self.shares_mock.get.return_value = self._share
-
         self.shares_mock.get_metadata.return_value = self._share.metadata
 
         # Get the command object to test
@@ -1779,7 +1888,6 @@ class TestShowShareProperties(TestShare):
             ("share", self._share.id)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
         columns, data = self.cmd.take_action(parsed_args)
         self.shares_mock.get.assert_called_with(self._share.id)
         self.shares_mock.get_metadata.assert_called_with(self._share)
@@ -1814,7 +1922,6 @@ class TestShareRevert(TestShare):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
-
         self.shares_mock.get.assert_called_with(self.share_snapshot.share_id)
         self.share.revert_to_snapshot.assert_called_with(self.share_snapshot)
         self.assertIsNone(result)
@@ -1831,3 +1938,233 @@ class TestShareRevert(TestShare):
         self.share.revert_to_snapshot.side_effect = Exception()
         self.assertRaises(
             osc_exceptions.CommandError, self.cmd.take_action, parsed_args)
+
+
+class TestShareMigrationStart(TestShare):
+
+    def setUp(self):
+        super(TestShareMigrationStart, self).setUp()
+
+        self.new_share_type = manila_fakes.FakeShareType.create_one_sharetype()
+        self.share_types_mock.get.return_value = self.new_share_type
+
+        self.new_share_network = manila_fakes.FakeShareNetwork \
+            .create_one_share_network()
+        self.share_networks_mock.get.return_value = self.new_share_network
+
+        self._share = manila_fakes.FakeShare.create_one_share(
+            attrs={
+                'status': 'available'},
+            methods={'migration_start': None})
+        self.shares_mock.get.return_value = self._share
+
+        self.shares_mock.manage.return_value = self._share
+
+        # Get the command objects to test
+        self.cmd = osc_shares.ShareMigrationStart(self.app, None)
+
+    def test_migration_start_with_new_share_type(self):
+        """Test with new_share_type"""
+        arglist = [
+            self._share.id,
+            'host@driver#pool',
+            '--preserve-metadata', 'False',
+            '--preserve-snapshots', 'False',
+            '--writable', 'False',
+            '--nondisruptive', 'False',
+            '--new-share-type', self.new_share_type.id,
+            '--force-host-assisted-migration', 'False'
+
+        ]
+        verifylist = [
+            ('share', self._share.id),
+            ('host', 'host@driver#pool'),
+            ('preserve_metadata', 'False'),
+            ('preserve_snapshots', 'False'),
+            ('writable', 'False'),
+            ('nondisruptive', 'False'),
+            ('new_share_type', self.new_share_type.id),
+            ('force_host_assisted_migration', 'False')
+
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self._share.migration_start.assert_called_with(
+            "host@driver#pool",
+            'False',
+            'False',
+            'False',
+            'False',
+            'False',
+            None,
+            self.new_share_type.id
+        )
+        self.assertIsNone(result)
+
+    def test_migration_start_with_new_share_network(self):
+        """Test with new_share_network"""
+
+        arglist = [
+            self._share.id,
+            'host@driver#pool',
+            '--preserve-metadata', 'False',
+            '--preserve-snapshots', 'False',
+            '--writable', 'False',
+            '--nondisruptive', 'False',
+            '--new-share-network', self.new_share_network.id,
+            '--force-host-assisted-migration', 'False'
+
+        ]
+        verifylist = [
+            ('share', self._share.id),
+            ('host', 'host@driver#pool'),
+            ('preserve_metadata', 'False'),
+            ('preserve_snapshots', 'False'),
+            ('writable', 'False'),
+            ('nondisruptive', 'False'),
+            ('new_share_network', self.new_share_network.id),
+            ('force_host_assisted_migration', 'False')
+
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self._share.migration_start.assert_called_with(
+            "host@driver#pool",
+            'False',
+            'False',
+            'False',
+            'False',
+            'False',
+            self.new_share_network.id,
+            None
+        )
+        self.assertIsNone(result)
+
+
+class TestShareMigrationCancel(TestShare):
+
+    def setUp(self):
+        super(TestShareMigrationCancel, self).setUp()
+
+        self._share = manila_fakes.FakeShare.create_one_share(
+            attrs={
+                'status': 'migrating'},
+            methods={'migration_cancel': None})
+        self.shares_mock.get.return_value = self._share
+
+        self.shares_mock.manage.return_value = self._share
+
+        # Get the command objects to test
+        self.cmd = osc_shares.ShareMigrationCancel(self.app, None)
+
+    def test_migration_cancel(self):
+        arglist = [
+            self._share.id
+        ]
+        verifylist = [
+            ('share', self._share.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self._share.migration_cancel.assert_called
+        self.assertIsNone(result)
+
+
+class TestShareMigrationComplete(TestShare):
+
+    def setUp(self):
+        super(TestShareMigrationComplete, self).setUp()
+
+        self._share = manila_fakes.FakeShare.create_one_share(
+            attrs={
+                'status': 'migrating'},
+            methods={'migration_complete': None})
+        self.shares_mock.get.return_value = self._share
+
+        self.shares_mock.manage.return_value = self._share
+
+        # Get the command objects to test
+        self.cmd = osc_shares.ShareMigrationComplete(self.app, None)
+
+    def test_migration_complete(self):
+        arglist = [
+            self._share.id
+        ]
+        verifylist = [
+            ('share', self._share.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self._share.migration_complete.assert_called
+        self.assertIsNone(result)
+
+
+class TestShareMigrationShow(TestShare):
+
+    def setUp(self):
+        super(TestShareMigrationShow, self).setUp()
+
+        self._share = manila_fakes.FakeShare.create_one_share(
+            attrs={
+                'status': 'available',
+                'task_state': 'migration_in_progress'},
+            methods={'migration_get_progress': ("<Response [200]>",
+                     {'total_progress': 0, 'task_state':
+                      'migration_in_progress',
+                      'details': {}})
+                     })
+        self.shares_mock.get.return_value = self._share
+
+        self.shares_mock.manage.return_value = self._share
+
+        # Get the command objects to test
+        self.cmd = osc_shares.ShareMigrationShow(self.app, None)
+
+    def test_migration_show(self):
+        arglist = [
+            self._share.id
+        ]
+        verifylist = [
+            ('share', self._share.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+        self._share.migration_get_progress.assert_called
+
+
+class TestShareRestore(TestShare):
+
+    def setUp(self):
+        super(TestShareRestore, self).setUp()
+
+        self.share = manila_fakes.FakeShare.create_one_share(
+            methods={'restore': None}
+        )
+        self.shares_mock.get.return_value = self.share
+        self.cmd = osc_shares.RestoreShare(self.app, None)
+
+        def test_share_restore(self):
+            arglist = [
+                self.share.name
+            ]
+            verifylist = [
+                ('share', [self.share.name])
+            ]
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+            result = self.cmd.take_action(parsed_args)
+
+            self.shares_mock.get.assert_called_with(self.share)
+            self.share.restore.assert_called_with(self.share)
+            self.assertIsNone(result)
+
+        def test_share_restore_exception(self):
+            arglist = [
+                self.share.name
+            ]
+            verifylist = [
+                ('share', [self.share.name])
+            ]
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+            self.share.restore.side_effect = Exception()
+            self.assertRaises(
+                osc_exceptions.CommandError, self.cmd.take_action, parsed_args)
